@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ *  Copyright (C) 2000-2011, Parallels, Inc. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,7 +19,10 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
+#include <mntent.h>
+#include <string.h>
 #include <sys/mount.h>
+#include <sys/param.h>
 
 #include "fs.h"
 #include "util.h"
@@ -30,21 +33,21 @@
 
 int vps_is_run(vps_handler *h, envid_t veid);
 
-/** Get VE mount status.
+/** Get CT mount status.
  *
- * @param root		VE root.
- * @return		 1 - VE mounted
- *			 0 - VE unmounted.
+ * @param root		CT root.
+ * @return		 1 - CT mounted
+ *			 0 - CT unmounted.
  *			-1 - error
  */
-int vps_is_mounted(char *root)
+int vps_is_mounted(const char *root)
 {
 	return vz_fs_is_mounted(root);
 }
 
-/** Mount VE.
+/** Mount CT.
  *
- * @param veid		VE id.
+ * @param veid		CT ID.
  * @param fs		file system parameters.
  * @param dq		disk quota parameters.
  * @return		0 on success.
@@ -65,53 +68,29 @@ int fsmount(envid_t veid, fs_param *fs, dq_param *dq)
 	return ret;
 }
 
-static int real_umount(envid_t veid, char *root)
-{
-	int i, ret, n;
-
-	n = 0;
-	for (i = 0; i < 2; i++) {
-		while (1) {
-			ret = umount2(root, MNT_DETACH);
-			if (ret < 0) {
-				if (n > 0 && errno == EINVAL)
-					ret = 0;
-				break;
-			}
-			n++;
-		}
-		if (ret == 0 || (ret < 0 && errno != EBUSY))
-			break;
-		sleep(1);
-	}
-	if (ret) {
-		logger(-1, errno, "Can't umount: %s", root);
-		ret = VZ_FS_CANTUMOUNT;
-	}
-	return ret;
-}
-
-/** Unmount VE.
+/** Unmount CT.
  *
- * @param veid		VE id.
- * @param root		VE root.
+ * @param veid		CT ID.
+ * @param root		CT root.
  * @return		0 on success.
  */
-int fsumount(envid_t veid, char *root)
+int fsumount(envid_t veid, const char *root)
 {
-	int ret;
-
-	if (!(ret = real_umount(veid, root))) {
-		if (!quota_ctl(veid, QUOTA_STAT))
-			ret = quota_off(veid, 0);
+	if (umount(root) != 0) {
+		logger(-1, errno, "Can't umount %s", root);
+		return VZ_FS_CANTUMOUNT;
 	}
-	return ret;
+
+	if (!quota_ctl(veid, QUOTA_STAT))
+		return quota_off(veid, 0);
+
+	return 0;
 }
 
-/** Mount VE and run mount action script if exists.
+/** Mount CT and run mount action script if exists.
  *
- * @param h		VE handler.
- * @param veid		VE id.
+ * @param h		CT handler.
+ * @param veid		CT ID.
  * @param fs		file system parameters.
  * @param dq		disk quota parameters.
  * @param skip		skip mount action scrips
@@ -128,16 +107,32 @@ int vps_mount(vps_handler *h, envid_t veid, fs_param *fs, dq_param *dq,
 	if (check_var(fs->private, "VE_PRIVATE is not set"))
 		return VZ_VE_PRIVATE_NOTSET;
 	if (!stat_file(fs->private)) {
-		logger(-1, 0, "VE private area %s does not exist", fs->private);
+		logger(-1, 0, "Container private area %s does not exist",
+				fs->private);
 		return VZ_FS_NOPRVT;
 	}
 	if (vps_is_mounted(fs->root)) {
-		logger(-1, 0, "VE is already mounted");
+		logger(-1, 0, "Container is already mounted");
 		return 0;
 	}
+	/* Execute pre mount scripts */
+	if (!(skip & SKIP_ACTION_SCRIPT)) {
+		snprintf(buf, sizeof(buf), "%svps.%s", VPS_CONF_DIR,
+			PRE_MOUNT_PREFIX);
+		for (i = 0; i < 2; i++) {
+			if (run_pre_script(veid, buf)) {
+				logger(-1, 0, "Error executing mount script %s",
+					buf);
+				fsumount(veid, fs->root);
+				return VZ_ACTIONSCRIPT_ERROR;
+			}
+			snprintf(buf, sizeof(buf), "%s%d.%s", VPS_CONF_DIR,
+				veid, PRE_MOUNT_PREFIX);
+		}
+	}
 	if ((ret = fsmount(veid, fs, dq)))
 		return ret;
-	/* Execute per VE & global mount scripts */
+	/* Execute per-CT & global mount scripts */
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
 		snprintf(buf, sizeof(buf), "%svps.%s", VPS_CONF_DIR,
 			MOUNT_PREFIX);
@@ -152,30 +147,59 @@ int vps_mount(vps_handler *h, envid_t veid, fs_param *fs, dq_param *dq,
 				veid, MOUNT_PREFIX);
 		}
 	}
-	logger(0, 0, "VE is mounted");
+	logger(0, 0, "Container is mounted");
 
 	return 0;
 }
 
-/** Unmount VE and run unmount action script if exists.
+static int umount_submounts(const char *root)
+{
+	FILE *fp;
+	struct mntent *mnt;
+	int len;
+	char path[MAXPATHLEN + 1];
+
+	if (realpath(root, path) == NULL) {
+		logger(-1, errno, "realpath(%s) failed", root);
+		return -1;
+	}
+	if ((fp = setmntent("/proc/mounts", "r")) == NULL) {
+		logger(-1, errno, "Unable to open /proc/mounts");
+		return -1;
+	}
+	strcat(path, "/"); /* skip base mountpoint */
+	len = strlen(path);
+	while ((mnt = getmntent(fp)) != NULL) {
+		if (strncmp(path, mnt->mnt_dir, len) == 0) {
+			if (umount(mnt->mnt_dir))
+				logger(-1, errno, "Cannot umount %s",
+						mnt->mnt_dir);
+		}
+	}
+	endmntent(fp);
+
+	return 0;
+}
+
+/** Unmount CT and run unmount action script if exists.
  *
- * @param h		VE handler.
- * @param veid		VE id.
- * @param root		VE root.
+ * @param h		CT handler.
+ * @param veid		CT ID.
+ * @param root		CT root.
  * @param skip		skip unmount action scrips
  * @return		0 on success.
  */
-int vps_umount(vps_handler *h, envid_t veid, char *root, skipFlags skip)
+int vps_umount(vps_handler *h, envid_t veid, const char *root, skipFlags skip)
 {
 	char buf[PATH_LEN];
 	int ret, i;
 
 	if (!vps_is_mounted(root)) {
-		logger(-1, 0, "VE is not mounted");
+		logger(-1, 0, "CT is not mounted");
 		return VZ_FS_NOT_MOUNTED;
 	}
 	if (vps_is_run(h, veid)) {
-		logger(-1, 0, "VE is running. Stop VE first");
+		logger(-1, 0, "Container is running -- stop it first");
 		return 0;
 	}
 	if (!(skip & SKIP_ACTION_SCRIPT)) {
@@ -191,8 +215,22 @@ int vps_umount(vps_handler *h, envid_t veid, char *root, skipFlags skip)
 				UMOUNT_PREFIX);
 		}
 	}
+	umount_submounts(root);
 	if (!(ret = fsumount(veid, root)))
-		logger(0, 0, "VE is unmounted");
+		logger(0, 0, "Container is unmounted");
+	if (!(skip & SKIP_ACTION_SCRIPT)) {
+		snprintf(buf, sizeof(buf), "%s%d.%s", VPS_CONF_DIR,
+			veid, POST_UMOUNT_PREFIX);
+		for (i = 0; i < 2; i++) {
+			if (run_pre_script(veid, buf)) {
+				logger(-1, 0, "Error executing umount script %s",
+					buf);
+				return VZ_ACTIONSCRIPT_ERROR;
+			}
+			snprintf(buf, sizeof(buf), "%svps.%s", VPS_CONF_DIR,
+				POST_UMOUNT_PREFIX);
+		}
+	}
 
 	return ret;
 }
@@ -206,7 +244,7 @@ int vps_set_fs(fs_param *g_fs, fs_param *fs)
 	if (check_var(g_fs->private, "VE_PRIVATE is not set"))
 		return VZ_VE_PRIVATE_NOTSET;
 	if (!vps_is_mounted(g_fs->root)) {
-		logger(-1, 0, "VE is not mounted");
+		logger(-1, 0, "Container is not mounted");
 		return VZ_FS_NOT_MOUNTED;
 	}
 	g_fs->noatime = fs->noatime;
