@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2007 SWsoft. All rights reserved.
+ * Copyright (C) 2000-2008, Parallels, Inc. All rights reserved.
  *
  * This code is public domain.
  *
@@ -34,6 +34,8 @@
 #define EXC_RECV	3
 #define EXC_NORECV	4
 
+#define NAME "ndsend: "
+
 char* iface = NULL;
 
 struct in6_addr src_ipaddr;
@@ -56,7 +58,7 @@ struct nd_packet {
 	__u8		obits[6];
 };
 
-int init_device_addresses(int sock, const char* device)
+static int init_device_addresses(int sock, const char* device)
 {
 	struct ifreq ifr;
 
@@ -64,14 +66,16 @@ int init_device_addresses(int sock, const char* device)
 	strncpy(ifr.ifr_name, device, IFNAMSIZ-1);
 
 	if (ioctl(sock, SIOCGIFINDEX, &ifr) != 0) {
-		fprintf(stderr, "unknown iface %s : %m", device);
+		fprintf(stderr, NAME "Unknown network interface %s: %m\n",
+				device);
 		return -1;
 	}
 	ifindex = ifr.ifr_ifindex;
 
 	/* get interface HW address */
 	if (ioctl(sock, SIOCGIFHWADDR, &ifr) != 0) {
-		fprintf(stderr, "can't get iface '%s' HW address : %m", device);
+		fprintf(stderr, NAME "Can not get the MAC address of "
+				"network interface %s: %m\n", device);
 		return -1;
 	}
 	memcpy(real_hwaddr, ifr.ifr_hwaddr.sa_data, 6);
@@ -82,7 +86,7 @@ int init_device_addresses(int sock, const char* device)
 int sock;
 struct nd_packet pkt;
 
-void create_nd_packet(struct nd_packet* pkt)
+static void create_nd_packet(struct nd_packet* pkt)
 {
 	pkt->icmp6_type = ND_NEIGHBOR_ADVERT;
 	pkt->icmp6_code = 0;
@@ -98,7 +102,7 @@ void create_nd_packet(struct nd_packet* pkt)
 	memcpy(&pkt->obits, real_hwaddr, 6);
 }
 
-void sender(void)
+static void sender(void)
 {
 	struct sockaddr_in6 to;
 
@@ -112,22 +116,37 @@ void sender(void)
 
 	if (sendto(sock, &pkt, sizeof(pkt), 0,
 	    (struct sockaddr*) &to, sizeof(to)) < 0) {
-		fprintf(stderr, "sendto : %m");
+		fprintf(stderr, NAME "Error in sendto(): %m\n");
 		exit(EXC_SYS);
 	}
 }
 
+static void usage()
+{
+	printf(
+"ndsend sends an unsolicited Neighbor Advertisement ICMPv6 multicast packet\n"
+"announcing a given IPv6 address to all IPv6 nodes as per RFC4861.\n\n"
+"Usage: ndsend <address> <interface> (example: ndsend 2001:DB8::1 eth0)\n\n"
+"Note: ndsend is called by arpsend -U -i, see arpsend(8) man page.\n"
+	);
+}
+
 int main(int argc,char** argv)
 {
 	int value;
 
+	if (argc != 3) {
+		usage();
+		exit(EXC_USAGE);
+	}
+
 	if (inet_pton(AF_INET6, argv[1], &src_ipaddr) <= 0)
 		return -1;
 	iface = argv[2];
 
 	sock = socket(PF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
 	if (sock < 0) {
-		fprintf(stderr, "socket : %m");
+		fprintf(stderr, NAME "Error in socket(): %m\n");
 		exit(EXC_SYS);
 	}
 
